@@ -221,20 +221,6 @@ impl LanguageTags {
 }
 
 impl UserLanguage {
-    pub fn new(language: &Language) -> UserLanguage {
-        match *language {
-            Language::FromEnvironment => UserLanguage::LanguageTags(
-                LanguageTags::from_locale(&locale_from_environment())
-                    .map_err(|s| {
-                        rsvg_log!("could not convert locale to language tags: {}", s);
-                    })
-                    .unwrap_or_else(|_| LanguageTags::empty()),
-            ),
-
-            Language::AcceptLanguage(ref a) => UserLanguage::AcceptLanguage(a.clone()),
-        }
-    }
-
     pub fn any_matches(&self, tags: &LanguageTags) -> bool {
         match *self {
             UserLanguage::LanguageTags(ref language_tags) => {
@@ -247,28 +233,6 @@ impl UserLanguage {
     }
 }
 
-/// Gets the user's preferred locale from the environment and
-/// translates it to a `Locale` with `LanguageRange` fallbacks.
-///
-/// The `Locale::current()` call only contemplates a single language,
-/// but glib is smarter, and `g_get_langauge_names()` can provide
-/// fallbacks, for example, when LC_MESSAGES="en_US.UTF-8:de" (USA
-/// English and German).  This function converts the output of
-/// `g_get_language_names()` into a `Locale` with appropriate
-/// fallbacks.
-fn locale_from_environment() -> Locale {
-    let mut locale = Locale::invariant();
-
-    for name in glib::language_names() {
-        let name = name.as_str();
-        if let Ok(range) = LanguageRange::from_unix(name) {
-            locale.add(&range);
-        }
-    }
-
-    locale
-}
-
 #[cfg(test)]
 mod tests {
     use super::*;
