@@ -11,6 +11,7 @@ use std::fmt;
 use std::include_str;
 use std::rc::Rc;
 use std::str::FromStr;
+use std::sync::Arc;
 
 use crate::css::{self, Origin, Stylesheet};
 use crate::error::{AcquireError, AllowedUrlError, LoadingError, NodeIdError};
@@ -18,6 +19,7 @@ use crate::handle::LoadOptions;
 use crate::io::{self, BinaryData};
 use crate::limits;
 use crate::node::{Node, NodeBorrow, NodeData};
+use crate::session::Session;
 use crate::surface_utils::shared_surface::SharedImageSurface;
 use crate::url_resolver::{AllowedUrl, UrlResolver};
 use crate::xml::{xml_load_from_possibly_compressed_stream, Attributes};
@@ -27,8 +29,9 @@ static UA_STYLESHEETS: Lazy<Vec<Stylesheet>> = Lazy::new(|| {
         include_str!("ua.css"),
         &UrlResolver::new(None),
         Origin::UserAgent,
+        Session::default(),
     )
-    .unwrap()]
+    .expect("could not parse user agent stylesheet for librsvg, there's a bug!")]
 });
 
 /// Identifier of a node
@@ -72,6 +75,9 @@ pub struct Document {
     /// Tree of nodes; the root is guaranteed to be an `<svg>` element.
     tree: Node,
 
+    /// Metadata about the SVG handle.
+    session: Session,
+
     /// Mapping from `id` attributes to nodes.
     ids: HashMap<String, Node>,
 
@@ -85,22 +91,23 @@ pub struct Document {
     images: RefCell<Images>,
 
     /// Used to load referenced resources.
-    load_options: LoadOptions,
+    load_options: Arc<LoadOptions>,
 
-    /// Stylesheets defined in the document
+    /// Stylesheets defined in the document.
     stylesheets: Vec<Stylesheet>,
 }
 
 impl Document {
     /// Constructs a `Document` by loading it from a stream.
     pub fn load_from_stream(
-        load_options: &LoadOptions,
+        session: Session,
+        load_options: Arc<LoadOptions>,
         stream: &gio::InputStream,
         cancellable: Option<&gio::Cancellable>,
     ) -> Result<Document, LoadingError> {
         xml_load_from_possibly_compressed_stream(
-            DocumentBuilder::new(load_options),
-            load_options.unlimited_size,
+            DocumentBuilder::new(session, load_options.clone()),
+            load_options,
             stream,
             cancellable,
         )
@@ -115,7 +122,8 @@ impl Document {
         let stream = gio::MemoryInputStream::from_bytes(&bytes);
 
         Document::load_from_stream(
-            &LoadOptions::new(UrlResolver::new(None)),
+            Session::new_for_test_suite(),
+            Arc::new(LoadOptions::new(UrlResolver::new(None))),
             &stream.upcast(),
             None::<&gio::Cancellable>,
         )
@@ -134,7 +142,7 @@ impl Document {
             NodeId::External(url, id) => self
                 .externs
                 .borrow_mut()
-                .lookup(&self.load_options, url, id)
+                .lookup(&self.session, &self.load_options, url, id)
                 .ok(),
         }
     }
@@ -159,8 +167,14 @@ impl Document {
     ///
     /// This uses the default UserAgent stylesheet, the document's internal stylesheets,
     /// plus an extra set of stylesheets supplied by the caller.
-    pub fn cascade(&mut self, extra: &[Stylesheet]) {
-        css::cascade(&mut self.tree, &UA_STYLESHEETS, &self.stylesheets, extra);
+    pub fn cascade(&mut self, extra: &[Stylesheet], session: &Session) {
+        css::cascade(
+            &mut self.tree,
+            &UA_STYLESHEETS,
+            &self.stylesheets,
+            extra,
+            session,
+        );
     }
 }
 
@@ -177,16 +191,18 @@ impl Resources {
 
     pub fn lookup(
         &mut self,
+        session: &Session,
         load_options: &LoadOptions,
         url: &str,
         id: &str,
     ) -> Result<Node, LoadingError> {
-        self.get_extern_document(load_options, url)
+        self.get_extern_document(session, load_options, url)
             .and_then(|doc| doc.lookup_internal_node(id).ok_or(LoadingError::BadUrl))
     }
 
     fn get_extern_document(
         &mut self,
+        session: &Session,
         load_options: &LoadOptions,
         href: &str,
     ) -> Result<Rc<Document>, LoadingError> {
@@ -204,7 +220,8 @@ impl Resources {
                     .map_err(LoadingError::from)
                     .and_then(|stream| {
                         Document::load_from_stream(
-                            &load_options.copy_with_base_url(aurl),
+                            session.clone(),
+                            Arc::new(load_options.copy_with_base_url(aurl)),
                             &stream,
                             None,
                         )
@@ -476,54 +493,74 @@ impl NodeStack {
     }
 }
 
+/// Used to build a tree of SVG nodes while an XML document is being read.
+///
+/// This struct holds the document-related state while loading an SVG document from XML:
+/// the loading options, the partially-built tree of nodes, the CSS stylesheets that
+/// appear while loading the document.
+///
+/// The XML loader asks a `DocumentBuilder` to
+/// [`append_element`][DocumentBuilder::append_element],
+/// [`append_characters`][DocumentBuilder::append_characters], etc.  When all the XML has
+/// been consumed, the caller can use [`build`][DocumentBuilder::build] to get a
+/// fully-loaded [`Document`].
 pub struct DocumentBuilder {
-    load_options: LoadOptions,
+    /// Metadata for the document's lifetime.
+    session: Session,
+
+    /// Loading options; mainly the URL resolver.
+    load_options: Arc<LoadOptions>,
+
+    /// Root node of the tree.
     tree: Option<Node>,
+
+    /// Mapping from `id` attributes to nodes.
     ids: HashMap<String, Node>,
+
+    /// Stylesheets defined in the document.
     stylesheets: Vec<Stylesheet>,
 }
 
 impl DocumentBuilder {
-    pub fn new(load_options: &LoadOptions) -> DocumentBuilder {
+    pub fn new(session: Session, load_options: Arc<LoadOptions>) -> DocumentBuilder {
         DocumentBuilder {
-            load_options: load_options.clone(),
+            session,
+            load_options,
             tree: None,
             ids: HashMap::new(),
             stylesheets: Vec::new(),
         }
     }
 
-    pub fn append_stylesheet_from_xml_processing_instruction(
-        &mut self,
-        alternate: Option<String>,
-        type_: Option<String>,
-        href: &str,
-    ) -> Result<(), LoadingError> {
-        if type_.as_deref() != Some("text/css")
-            || (alternate.is_some() && alternate.as_deref() != Some("no"))
-        {
-            return Err(LoadingError::Other(String::from(
-                "invalid parameters in XML processing instruction for stylesheet",
-            )));
-        }
-
-        // FIXME: handle CSS errors
-        if let Ok(stylesheet) =
-            Stylesheet::from_href(href, &self.load_options.url_resolver, Origin::Author)
-        {
-            self.stylesheets.push(stylesheet);
-        }
-
-        Ok(())
+    pub fn session(&self) -> &Session {
+        &self.session
     }
 
+    /// Adds a stylesheet in order to the document.
+    ///
+    /// Stylesheets will later be matched in the order in which they were added.
+    pub fn append_stylesheet(&mut self, stylesheet: Stylesheet) {
+        self.stylesheets.push(stylesheet);
+    }
+
+    /// Creates an element of the specified `name` as a child of `parent`.
+    ///
+    /// This is the main function to create new SVG elements while parsing XML.
+    ///
+    /// `name` is the XML element's name, for example `rect`.
+    ///
+    /// `attrs` has the XML element's attributes, e.g. cx/cy/r for `<circle cx="0" cy="0"
+    /// r="5">`.
+    ///
+    /// If `parent` is `None` it means that we are creating the root node in the tree of
+    /// elements.  The code will later validate that this is indeed an `<svg>` element.
     pub fn append_element(
         &mut self,
         name: &QualName,
         attrs: Attributes,
         parent: Option<Node>,
     ) -> Node {
-        let node = Node::new(NodeData::new_element(name, attrs));
+        let node = Node::new(NodeData::new_element(&self.session, name, attrs));
 
         if let Some(id) = node.borrow_element().get_id() {
             // This is so we don't overwrite an existing id
@@ -543,38 +580,28 @@ impl DocumentBuilder {
         node
     }
 
-    pub fn append_stylesheet_from_text(&mut self, text: &str) {
-        // FIXME: handle CSS errors
-        if let Ok(stylesheet) =
-            Stylesheet::from_data(text, &self.load_options.url_resolver, Origin::Author)
-        {
-            self.stylesheets.push(stylesheet);
-        }
-    }
-
+    /// Creates a node for an XML text element as a child of `parent`.
     pub fn append_characters(&mut self, text: &str, parent: &mut Node) {
         if !text.is_empty() {
-            self.append_chars_to_parent(text, parent);
+            // When the last child is a Chars node we can coalesce
+            // the text and avoid screwing up the Pango layouts
+            if let Some(child) = parent.last_child().filter(|c| c.is_chars()) {
+                child.borrow_chars().append(text);
+            } else {
+                parent.append(Node::new(NodeData::new_chars(text)));
+            };
         }
     }
 
-    fn append_chars_to_parent(&mut self, text: &str, parent: &mut Node) {
-        // When the last child is a Chars node we can coalesce
-        // the text and avoid screwing up the Pango layouts
-        if let Some(child) = parent.last_child().filter(|c| c.is_chars()) {
-            child.borrow_chars().append(text);
-        } else {
-            parent.append(Node::new(NodeData::new_chars(text)));
-        };
-    }
-
     pub fn resolve_href(&self, href: &str) -> Result<AllowedUrl, AllowedUrlError> {
         self.load_options.url_resolver.resolve_href(href)
     }
 
+    /// Does the final validation on the `Document` being read, and returns it.
     pub fn build(self) -> Result<Document, LoadingError> {
         let DocumentBuilder {
             load_options,
+            session,
             tree,
             ids,
             stylesheets,
@@ -586,6 +613,7 @@ impl DocumentBuilder {
                 if is_element_of_type!(root, Svg) {
                     let mut document = Document {
                         tree: root,
+                        session: session.clone(),
                         ids,
                         externs: RefCell::new(Resources::new()),
                         images: RefCell::new(Images::new()),
@@ -593,7 +621,7 @@ impl DocumentBuilder {
                         stylesheets,
                     };
 
-                    document.cascade(&[]);
+                    document.cascade(&[], &session);
 
                     Ok(document)
                 } else {
